/*
gpsParserDefines.h 
by TJ Johnson
3/22/09

This diver uses 2 cogs, 1 for FDserial and the other to sort and process the 
gps data.

Call gpsStart(Pin) to initiate the driver.

Then you can call any of the functions for the specific piece of data 
needed.

If there is a conflict with the defines there is a non define version included 
in the package

All define based functions start with "gps" then the data name the data name is 
the 1st letter capatilized. 

Define based functions
Information           Function           Return Data Type
Valid Signal		  gpsValid     		 int
Satalites             gpsSats            int
Date Stamp            gpsDate            str pointer
Time Stamp            gpsTime            str pointer
Altitude              gpsAlt           	 int
Altitude Tenths       gpsAltTenths     	 int
Altitude              gpsAltFloat      	 float
Heading	 		      gpsHeading       	 int
Heading Tenths        gpsHeadingTenths 	 int
Heading			      gpsHeadingFloat  	 float
Speed			      gpsSpeed         	 int
Speed Tenths          gpsSpeedTenths   	 int
Speed                 gpsSpeedFloat    	 float
Latitude		      gpsLat		     str pointer
Latitude Deg          gpsLatDeg        	 int +is North - is south
Latitude Min          gpsLatMin        	 int
Latitude Sec          gpsLatSec        	 int
Latitude Decimal Min  gpsLatDecMin     	 float
Longitude             gpsLong          	 str pointer
Longitude Deg         gpsLongDeg       	 int + is east - is west
Longitude Min         gpsLongMin       	 int
Longitude Sec         gpsLongSec       	 int
Longitude Decimal Min gpsLongDecMin      float
        


Non Define based data. These functions require additional cycles and may not 
want to be used in a time critical system

Information           Function           Return Data Type
Time Stamp      	  getGpsTime()          str pointer
Hours UTC             getGpsTimeHours()     int
Min UTC               getGpsTimeMin()       int
Seconds UTC           getGpsTimeSec()       int
Day	 		          getGpsDay()           int
Month			      getGpsMonth()         int
Year                  getGpsYear()          int


The standard int functions for Altitude, Heading, Speed return a truncated
result. The tenths function have been provided to get the remainder. 
Example Heading is 163.4
       getHeading() returns 163
	   getHeadngTenths() returns 4
	    

For use with the Parallax GPS unit /raw pin should be connected to ground. 
WARNING : A 1k resistor must be placed in line with the SIO signal, 
 		  when connecting the Parallax GPS unit.		
		
WARNING : If the GPS Signal is invalid, there is a high chance that the altitude 
result will return trash, please protect your program accordingly.
	
	
See end of file for MIT License.		
*/
#ifndef __GPS_DEFINE_PARSER_H_
#define __GPS_DEFINE_PARSER_H_


#include <propeller.h>
#include <string.h>
#include <STDLIB.H>
#include "FdSerial.h"

 

// Global Static Constants

//Define based functions

#define gpsValid (valid-'0') // Convert the ascii valid char to a int

#define gpsTime (time) // Return the ponter to the string time.
	//Time Str format HHMMSS

#define gpsDateStr (date) // Return pointer to the date array
	//Date Format DDMMYY  

#define gpsLatStr lat // Return pointer to lat string
         
#define gpsLatDeg (getDeg(lat)) // Return the value of of lat degs as an int

#define gpsLatMin (getMin(lat)) //Return the value of lat min as an int

#define gpsLatSec (getDecSec(lat)) // Return lat seconds as an int
    
#define gpsLatDecMin (getDecMin(lat)) // Return the decimal minutes 
// Decimal Minute Example 52.4544'
  
#define gpsLongStr (lon) // Return pointer to Long String

#define gpsLongDeg (getDeg(lon)) // Return Long Degrees

#define gpsLongMin (getMin(lon)) // Return Long Min
  
#define gpsLongSec (getDecSec(lon)) //Return Long Seconds  
  
#define gpsLongDecMin (getDecMin(lon)) // Return long dec minutes  
// Dec min example 52.5466

#define gpsAlt (atoi(alt)) // Return alt as an interger ( unit Meters)

#define gpsAltTenths (alt[6]-'0') // Return the 10ths of the altitude     
 
#define gpsAltFloat (atof(alt)) // Return altitude as a float
    
#define gpsHeadingDeg (atoi(heading)) // Return interger value of heading
  
#define gpsHeadingTenths (heading[4]-'0') // return the tenths of the heading

#define gpsHeadingFloat (atof(heading)) // Return heading as a float

#define gpsSats (sats[1] - '0') // Return current number of satilites in view
    
#define gpsSpeed (atoi(speed))  // Return speed as an int unit Knots
  
#define gpsSpeedTenths (speed[4]-'0') // return 10ths of speed
  
#define gpsSpeedFloat (atof(speed)) // return speed as float
  




// function prototypes

   // Name: gpsStart
   // Input: input pin
   // Output: cogId the parser is running in
   // Dependices: FdSerial.c
   // Process: Start a string parsing system that updates based on incomming 
   	  // gps data, use get****** to get a sepecific value.
   int gpsStart(const int Pin);
  
   // Name: gpsStop
   // Dependicies: none
   // Process: Stops the driver frees a cog.
   void gpsStop(void);

   // Name: getData
   // Dependicies: decodeGPRMC, decodeGPGGA, FdSerial_rx
   // Process: Starts a loop looking for the string ID and calls appropiate 
   	  //decode funciton  
   static void getData(void);

   // Name: decodeGPRMC
   // Dependicies: FdSerial_rx()
   // Process: parses throught the incomming serial data sorting to the correct 
   	  // data field  
   static void decodeGPRMC(void);

   // Name: decodeGPGGA
   // Dependicies: FdSerial_rx
   // Process: parses throught the incomming serial data sorting to the correct 
   	  // data field   
   static void decodeGPGGA(void);
   
   // Name: getTimeHours
   // Input: none
   // Output: The hours of current UTC time 
   // Dependicies: none
   // Process: converts the hours part of the time string to an int 
   int getGpsTimeHours(void);

   // Name: getGpsTimeMin
   // Input:none
   // Output:The minutes of current UTC time.
   // Dependicies: none
   // Process: convert the minute part of the time string to an int 
   int getGpsTimeMin(void);
  
   // Name: getGpsTimeSec
   // Input: none
   // Output: The seconds of current UTC time
   // Dependicies: none
   // Process: convert the minute part of the time string to an int
   int getGpsTimeSec(void);  

   // Name: getGpsDay
   // Input: none
   // Output: day of the timeStamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the day from the dateStamp. 
   int getGpsDay(void);
 
   // Name: getGpsMonth
   // Input: none
   // Output: month of timestamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the month from the dateStamp. 
   int getGpsMonth(void); 
   
   // Name: getGpsYear
   // Input: none
   // Output: year of timeStamp as an int.
   // Dependicies: none
   // Process: Returns interger value of the year from the dateStamp. 
   int getGpsYear(void);
   
   // Name: getGpsDeg
   // Input: char array 
   // Output: the current Deg as an int
   // Dependicies: atoi, charloc
   // Process: finds the decimal place in the string and works backwards to 
   	  //find the degrees
    int getDeg(char *array);

   // Name: getGpsMin
   // Input: char array
   // Output: the current minutes as an int
   // Dependicies: atoi,charLoc
   // Process: finds the minutes in the string working backwards from the
   	  // decimal place
    int getMin(char *array);

   // Name: getGpsDecMin
   // Input: char array 
   // Output: the floating point value of the decimal minutes
   // Dependicies: charLoc, atof
   // Process: removes the deg from the string and converts the remainder
   	  // to a float 
   float getDecMin(char *array);

   // Name: getGpsDecSec
   // Input: char array
   // Output: The interger of the seconds  
   // Dependicies: charLoc, atoi
   // Process: converts the remainder after the decimal to an interger 
   	  //and devides by 60 to convert to seconds 
    int getDecSec(char *array);

   // Name: charLoc
   // Input: char array, char to find
   // Output: the index of the char to find
   // Dependicies: none
   // Process: looks through the array for the seach char returning the index. 
   static int charLoc(char *array, char searchChar);

   #endif
/*
+------------------------------------------------------------------------------------------------------------------------------+
                                                   TERMS OF USE: MIT License                                                                                                              
+------------------------------------------------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         
+------------------------------------------------------------------------------------------------------------------------------+
*/